<?php
namespace Getresponse\WordPress;

defined( 'ABSPATH' ) || exit;

/**
 * Class WooCommerceExporter
 * @package Getresponse\WordPress
 */
class WooCommerceExporter {

	/** @var Api */
	private $api;

	/**
	 * @param Api $api
	 */
	public function __construct( $api ) {
		$this->api = $api;
	}

	/**
	 * @param $customerIds
	 * @param $campaign_id
	 * @param $autoresponder_id
	 * @param $customs
	 * @param $store_id
	 * @param $use_schedule
	 *
	 * @throws \Exception
	 */
	public function export_customers(
		$customerIds,
		$campaign_id,
		$autoresponder_id,
		$customs,
		$store_id,
		$use_schedule
	) {

		foreach ( $customerIds as $customer_id ) {

			try {
				$export_customer = ExportCustomerFactory::create_from_params(
					$campaign_id,
					$customer_id,
					$customs,
					$autoresponder_id,
					$store_id
				);

				if ( $use_schedule ) {

					$schedule_job_service = new ScheduleJobService(
						new ScheduleJobRepository(),
						new Configuration()
					);

					$schedule_job_service->add_schedule_job(
						ScheduleJob::EXPORT_CUSTOMER,
						$export_customer
					);

				} else {
					$this->export_customer( $export_customer );
				}
			} catch ( ApiException $e ) {
			}
		}
	}

	/**
	 * @param $export_customer ExportCustomer
	 *
	 * @return bool
	 * @throws \Exception
	 * @throws ApiException
	 */
	public function export_customer( $export_customer ) {

		$customer = new \WC_Customer( $export_customer->get_contact_id() );

		$orders = wc_get_orders( array(
			'meta_key'   => '_customer_user',
			'meta_value' => $export_customer->get_contact_id(),
		) );

		// if orders not found - don't send to GetResponse
		if ( empty( $orders ) ) {
			return false;
		}

		$customer_service = new CustomerService( $this->api );

		$customs = array();
		$data    = $customer->get_data();

		foreach ( $export_customer->get_custom_fields() as $woo_custom_name => $gr_custom_name ) {
			if ( ! empty( $data['billing'][ $woo_custom_name ] ) ) {
				$customs[ $gr_custom_name ] = $data['billing'][ $woo_custom_name ];
			}
		}

		try {
			$gr_customer = $customer_service->add_contact(
				$export_customer->get_campaign_id(),
				$customer->get_first_name(),
				$customer->get_email(),
				$export_customer->get_autoresponder_id(),
				$customs
			);
		} catch ( ContactAlreadyExistsException $e ) {
			$gr_customer = $customer_service->get_customer( $customer->get_email(),
				$export_customer->get_campaign_id() );
		}

		$store_id = $export_customer->get_store_id();

		// if store id is not set - don't import ecommerce details
		if ( empty( $store_id ) ) {
			return false;
		}

		if ( isset( $gr_customer['contactId'] ) && ! empty( $gr_customer['contactId'] ) ) {
			foreach ( $orders as $order ) {
			    try {
                    $this->export_order($order, $store_id, $gr_customer['contactId']);
                } catch (ApiException $e) {}
			}
		}

		return true;
	}

	/**
	 * @param \WC_Order $order
	 * @param string $store_id
	 * @param string $contact_id
	 *
	 * @throws ApiException
	 */
	public function export_order( $order, $store_id, $contact_id ) {

		$cart_service  = new CartService( $this->api );
		$order_service = new OrderService( $this->api );

		$this->upsert_order_products( $store_id, $order );

		$orders_map  = new OrdersMap();
		$gr_order_id = $orders_map->get_gr_order_id(
			$store_id,
			$order->get_id()
		);

		if ( empty( $gr_order_id ) ) {

			$cart_id = $cart_service->generate_cart_id();
			$cart = CartFactory::create_from_order(
				$store_id,
				$contact_id,
				$order,
				$cart_id
			);

			$cart_service->create_gr_cart( $store_id, $cart->for_api() );

		} else {
			$gr_order = $order_service->get_gr_order( $store_id, $gr_order_id );
			$cart_id  = $gr_order['cartId'];
		}

		$order_service->upsert_order(
			OrderFactory::create_from_params(
				$store_id,
				$cart_id,
				$contact_id,
				$order->get_id(),
                true
			)
		);
	}

	/**
	 * @param string $store_id
	 * @param \WC_Order $order
	 *
	 * @throws ApiException
	 */
	private function upsert_order_products( $store_id, $order ) {

		$product_service = new ProductService( $this->api );
		$product_factory = new \WC_Product_Factory();
		$product_map = new ProductsMap();

		/** @var \WC_Order_Item_Product $item */
		foreach ( $order->get_items() as $item ) {

			$gr_product_id = $product_map->get_gr_product_id( $store_id, $item->get_product_id() );

			/** @var \WC_Product $product */
			$wcProduct = $product_factory->get_product($item->get_product_id());

			if (!empty($gr_product_id)) {
                $grProduct = ProductFactory::buildFromSimpleProduct($wcProduct);
				$product_service->update_product($store_id, $gr_product_id, $grProduct->to_array());
			} else {
				$grProduct = ProductFactory::buildFromSimpleProduct($wcProduct);
				$product_service->create_product($store_id, $grProduct->to_array(), $wcProduct->get_id());
			}
		}
	}
}
